import json
import alpaca_trade_api as tradeapi
import os

"""
This function let's you interact 
with the Alpaca trade API 
"""

KEY_FILE = ""  # user key file path
APCA_API_KEY_ID = ""
APCA_API_SECRET_KEY = ""


def authentication_header():
    """ get's key and returns key in json format """
    with open(KEY_FILE, "r") as file:
        header = json.load(file)
    return header


def api():
    """ returns object for api """
    global KEY_FILE
    global APCA_API_KEY_ID
    global APCA_API_SECRET_KEY

    if "APCA_ID" in os.environ:
        APCA_ID = os.environ["APCA_ID"]
        APCA_KEY = os.environ["APCA_KEY"]
    elif KEY_FILE:
        auth_header = authentication_header()
        APCA_ID = str(auth_header["APCA-API-KEY-ID"])
        APCA_KEY = str(auth_header["APCA-API-SECRET-KEY"])
    else:
        APCA_ID = APCA_API_KEY_ID
        APCA_KEY = APCA_API_SECRET_KEY

    # Open the API connection
    api = tradeapi.REST(APCA_ID, APCA_KEY, "https://paper-api.alpaca.markets")
    # Get account info
    api.get_account()
    return api



from . import alpaca
from . import util
from . import stock


class Portfolio:
    """
    The class Portfolio, is for
    interacting with your own protfolio.
    """

    def __init__(self):
        self._equity = 0.0
        self._equity_yesterday = 0.0
        self._equity_change = 0.0
        self._cash = 0.0
        self._buying_power = 0.0
        self._list_orders = []
        self._owned_stocks = []
        self._position = []
        self._account = {}


    @property
    def orders(self):
        """ returns a list of all open orders with all diffult args """
        self._list_orders = alpaca.api().list_orders()
        return self._list_orders


    @property
    def position(self):  # PREV: get_position # TODO: change to positions
        """ create a list of all owned position """
        portfolio = alpaca.api().list_positions()
        self._position = []
        for position in portfolio:
            position_dict = util.reformat_position(position)
            position_dict["symbol"] = position.symbol
            self._position.append(position_dict)
        return self._position


    def owned_stock_qty(self, stock):  # maby shuld be in stock.Stock
        """ returns quantity of owned of a stock Stock (obj) """
        position = stock.position
        return position["qty"]


    @property
    def owned_stocks(self):
        """ returns a list of owned stocks """
        lst = self.position
        self._owned_stocks = []
        for dict_ in lst:
            self._owned_stocks.append(stock.Stock(dict_["symbol"]))
        return self._owned_stocks


    def sell_list(self, lst):
        """ takes a list of Stocks and sells all stocks in that list """
        owned_stocks = self.owned_stocks
        for stock_ in lst:
            # if stock_ in owned_stocks:
            qty = self.owned_stock_qty(stock_)
            # if not stock.symbol == 'GOOGL':
            # # BUG: fix, google has problem selling!
            stock_.sell(qty)


    @property
    def account(self):
        """ returns the dict of user account details"""
        self._account = util.reformat_position(alpaca.api().get_account())
        return self._account


    @property
    def buying_power(self):
        """ returns the amount of current buying_power that the user have"""
        self._buying_power = self.account["buying_power"]
        return self._buying_power


    def is_blocked(self):
        """ checks if the users has been blocked from trading """
        return self.account["trading_blocked"]


    @property
    def cash(self):
        """ returns the amount of a available liquid chash in account """
        self._cash = self.account["cash"]
        return self._cash


    @property
    def equity(self):
        """ returns the amount of equity that users has """
        self._equity = self.account["equity"]
        return self._equity


    @property
    def equity_yesterday(self):
        """ returns the amount of equity that was
        available at market close yesterday """
        self._equity_yesterday = self.account["last_equity"]
        return self._equity_yesterday


    @property
    def equity_change(self):
        """ returns the change in equity from yesterday to now """
        self._equity_change = self.equity - self.equity_yesterday
        return self._equity


    def __repr__(self):
        return f"portfolio({self.equity})"


    def __str__(self):
        return f"{self.position}"



import time
from . import alpaca
from . import stock



class Exchange:
    """
    This is the class instence of the Exchange.
    This class is used for interaction with the exchanges,
    for exampel NYSE. The exchange returns data and list of Stocks.
    """
    def __init__(self):
        self.name = ""
        self.exchanges = [
            "NASDAQ",
            "NYSE",
            "ARCA",
            "BATS",
        ]  # list of stock exchanges that is supported by alpaca
        self._is_open = False
        self._assets = []
        self._symbols = []  # ["sym", ... ]
        self._stocks = []  # [Stock(sym), ... ] obj
        self._historical_data = {}
        self._calendar = {}


    @property
    def is_open(self) -> bool:
        """ returns if exchange is open """
        self._is_open = alpaca.api().get_clock().is_open
        return self._is_open


    def calendar(self, start='2018-12-01', end='2018-12-01'):
        self._calendar = alpaca.api().get_calendar(start=start, end=end)[0].__dict__["_raw"]
        return self._calendar


    def assets_raw(self):
        """ returns a list of all avilabel stocks in exchanges list """
        all_assets = []
        active_assets = alpaca.api().list_assets(status="active")
        for exchange in self.exchanges:
            all_assets += [a for a in active_assets if a.exchange == exchange]
        self._assets = all_assets
        return self._assets


    @property
    def symbols(self):
        """ returns a list of all symbols """
        self._symbols = []
        for asset in self.assets_raw():
            self._symbols.append(asset.__dict__["_raw"]["symbol"])
        return self._symbols


    @property
    def stocks(self):
        """ returns a list of objects Stocks """
        self._stocks = []
        for sym in self.symbols:
            self._stocks.append(stock.Stock(sym))
        return self._stocks


    @property
    def historical_data(self):
        """ gathers all historical data on all stocks, {"sym":[data]} """
        self._historical_data = {}
        for stk in self.stocks:
            self._historical_data[stk.symbol] = stk.historical_data()
        return self._historical_data



"""
Cira

A simpler libray for alpaca-trade-api from Alpaca Markets.
"""

import alpaca_trade_api as tradeapi

from . import alpaca
from . import config
from . import util
from . import logging
from .exchange import Exchange
from .stock import Stock
from .portfolio import Portfolio

__version__ = "2.3.0"
__author__ = "Axel Gard"
__credits__ = "alpaca.markets"



def reformat_position(position):
    """ reformat position to be float values """
    raw_position = vars(position)["_raw"]
    position_dict = {}
    for key, value in raw_position.items():
        try:
            if isinstance(value, str):
                if "." in value:
                    position_dict[key] = float(value)
                else:
                    position_dict[key] = int(value)
        except ValueError:
            continue
    return position_dict



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='cira',
    version='2.3.0',
    description='A simpler library for the alapaca trade api',
    url='https://github.com/AxelGard/cira',
    author='Axel Gard',
    author_email='axel.gard@tutanota.com',
    license='MIT',
    packages=['cira'],
    long_description=long_description,
    long_description_content_type="text/markdown",

    install_requires=['alpaca-trade-api'],

    extras_requires = {
        "dev": [
            "pytest"
        ]
    },


    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Topic :: Office/Business :: Financial",
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)



# import alpaca_trade_api as tradeapi
import datetime
from . import config
from . import alpaca
from . import logging
from . import util


class Stock:
    """
    This is the class instence of a Stock.
    This class is for interaction of a Stock
    """
    def __init__(self, symbol:str):
        self.symbol = symbol
        self._price = 0
        self._value = 0
        self._is_shortable = False
        self._can_borrow = False
        self._barset = None
        self._week_pl_change = 0
        self._is_tradable = False
        self._position = None
        self._today_plpc = 0
        self._plpc = 0
        self._is_open = False


    @property
    def price(self) -> float:  # PREV: current_price
        """ returns the current price of given symbol (str) """
        if not self.exchange_is_open:
            self._price = self.value
        else:
            # OBS: due to API change no diffrence btween price and value
            self._price = self.barset(1)[0].c
        return self._price


    @property
    def value(self) -> float:  # prev: value_of_stock
        """ takes a string sym. Gets and returns the stock value at close """
        nr_days = 1
        bars = self.barset(nr_days)
        if bars is None:
            self._value = 0.0
        else:
            self._value = bars[0].c  # get stock at close
        return self._value


    def buy(self, qty: int):
        """ buys a stock. Takes int qty and a string sym """
        order_ = self.order(qty, "buy")
        if config.IS_LOGGING:
            logging.log(logging.format_log_action("buy", self.symbol, qty))
        return order_


    def sell(self, qty: int):
        """ sells a stock. Takes int qty and a string sym"""
        order_ = self.order(qty, "sell")
        if config.IS_LOGGING:
            logging.log(logging.format_log_action("sell", self.symbol, qty))
        return order_


    def order(self, qty: int, beh: str) -> float:
        """ submit order and is a template for order """
        if not self.is_tradable:
            raise Exception(f"Sorry, {self.symbol} is currantly not tradable on https://alpaca.markets/")
        else:
            order = alpaca.api().submit_order(
                symbol=self.symbol, qty=qty, side=beh,
                type="market", time_in_force="gtc"
            )
            return order


    @property
    def is_shortable(self) -> bool:
        """ checks if stock can be shorted """
        self._is_shortable = alpaca.api().get_asset(self.symbol).shortable
        return self._is_shortable


    @property
    def can_borrow(self) -> bool:
        """check whether the name is currently
        available to short at Alpaca"""
        self._can_borrow = alpaca.api().get_asset(self.symbol).easy_to_borrow
        return self._can_borrow


    def barset(self, limit:int):
        """ returns barset for stock for time period lim """
        self._barset = alpaca.api().get_barset(self.symbol, "minute", limit=int(limit))[self.symbol]
        return self._barset


    def historical_data(self, nr_days=365):
        """returns a list of the stocks closing value,
        range of 1 to 1000 days"""
        lst = []
        nr_days = max(1, min(nr_days, 1000))
        for bar in self.barset(nr_days):
            data = {
                "date": bar.t.strftime('%Y-%m-%d'),
                "open": bar.o,
                "high": bar.h,
                "low": bar.l,
                "close": bar.c
            }
            lst.append(data)
        return lst


    @property
    def week_pl_change(self) -> float:
        """ Percentage change over a week """
        nr_days = 5
        bars = self.barset(nr_days)
        week_open = bars[0].o
        week_close = bars[-1].c
        self._week_pl_change = (week_close - week_open) / week_open
        return self._week_pl_change


    @property
    def is_tradable(self) -> bool:
        """ return if the stock can be traded  """
        self._is_tradable = alpaca.api().get_asset(self.symbol).tradable
        return self._is_tradable


    @property
    def position(self):
        """ returns position of stock """
        pos = alpaca.api().get_position(self.symbol)
        self._position = util.reformat_position(pos)
        return self._position


    @property
    def today_plpc(self) -> float:
        """ stock today's profit/loss percent """
        self._today_plpc = self.position[
            "unrealized_intraday_plpc"
        ]
        return self._today_plpc


    @property
    def plpc(self) -> float:
        """ stock sym (str) Unrealized profit/loss percentage """
        self._plpc = self.position["unrealized_plpc"]
        return self._plpc


    @property
    def exchange_is_open(self) -> bool:
        """ returns if exchange is open """
        self._is_open = alpaca.api().get_clock().is_open
        return self._is_open


    def __repr__(self):
        return f"{self.symbol}@(${self.price})"


    def __str__(self):
        return f"{self.symbol}"

    # Operators

    def __eq__(self, other):
        if isinstance(other,(int,float)):
            return self.price == other
        return self.price == other.price


    def __ne__(self, other):
        if isinstance(other,(int,float)):
            return self.price != other
        return self.price != other.price


    def __lt__(self, other):
        if isinstance(other,(int,float)):
            return self.price < other
        return self.price < other.price


    def __le__(self, other):
        if isinstance(other,(int,float)):
            return self.price <= other
        return self.price <= other.price


    def __gt__(self, other):
        if isinstance(other,(int,float)):
            return self.price > other
        return self.price > other.price


    def __ge__(self, other):
        if isinstance(other,(int,float)):
            return self.price >= other
        return self.price >= other.price

    # Arithmetic Operators

    def __add__(self, other):
        if isinstance(other,(int,float)):
            return self.price + other
        return self.price + other.price


    def __radd__(self, other):
        return self.price + other


    def __sub__(self, other):
        if isinstance(other,(int,float)):
            return self.price - other
        return self.price - other.price


    def __rsub__(self, other):
        return self.price - other


    def __mul__(self, other):
        if isinstance(other,(int,float)):
            return self.price * other
        return self.price * other.price


    def __rmul__(self, other):
        return self.price * other


    def __truediv__(self, other):
        if isinstance(other,(int,float)):
            return self.price / other
        return self.price / other.price


    def __rdiv__(self, other):
        return self.price / other


    def __floordiv__(self, other):
        if isinstance(other,(int,float)):
            return self.price // other
        return self.price // other.price


    def __rfloordiv__(self, other):
        return self.price // other

    # Type Conversion

    def __abs__(self):
        # dose not rely makes sense should not be able to
        # be neg but might be good to have
        return abs(self.price)


    def __int__(self):
        return int(self.price)


    def __float__(self):
        return float(self.price)


    def __round__(self, nDigits):
        return round(self.price, nDigits)
